"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const simple_maybe_1 = require("simple-maybe");
const noop = () => { };
const truthOr = (x) => Array.isArray(x) && x.length ? x.some((a) => a) : false;
const truthAnd = (x) => Array.isArray(x) && x.length ? x.every((a) => a) : false;
const truthXor = (x) => Array.isArray(x) && x.length ? new Set(x).size > 1 : false;
const truthNor = (x) => Array.isArray(x) && x.length ? x.every((a) => !a) : false;
const Truth = (x) => ({
    map: (f) => Truth(f(x)),
    chain: (f) => f(x),
    ap: (y) => y.map(x),
    inspect: () => `Truth(${x})`,
    join: () => x,
    emit: () => x,
    concat: (o) => o.chain((r) => Truth(x.concat(r))),
    head: () => Array.isArray(x) && x.length ? x[0] : [],
    tail: () => Array.isArray(x) && x.length ? x[x.length - 1] : [],
    isEmpty: () => Boolean(!Array.isArray(x) || x.length === 0),
    or: () => truthOr(x),
    and: () => truthAnd(x),
    nor: () => truthNor(x),
    xor: () => truthXor(x),
    forkOr: (f, g) => (truthOr(x) ? g() : f()),
    forkOrL: (f) => (truthOr(x) ? noop() : f()),
    forkOrR: (f) => (truthOr(x) ? f() : noop()),
    forkAnd: (f, g) => (truthAnd(x) ? g() : f()),
    forkAndL: (f) => (truthAnd(x) ? noop() : f()),
    forkAndR: (f) => (truthAnd(x) ? f() : noop()),
    forkXor: (f, g) => (truthXor(x) ? g() : f()),
    forkXorL: (f) => (truthXor(x) ? noop() : f()),
    forkXorR: (f) => (truthXor(x) ? f() : noop()),
    forkNor: (f, g) => (truthNor(x) ? g() : f()),
    forkNorL: (f) => (truthNor(x) ? noop() : f()),
    forkNorR: (f) => (truthNor(x) ? f() : noop())
});
const truthTypeError = (x) => {
    console.error('Decision must be passed parameters that adhere to the documented type. Value that was passed:', x);
    return Truth([false]);
};
// @todo make much stricter; more of a sniff test right now
const TruthOf = (x) => Array.isArray(x) ? Truth(x) : truthTypeError(x);
const exportTruth = {
    of: TruthOf
};
exports.Truth = exportTruth;
const xFnTableFirst = (x) => simple_maybe_1.Maybe.of(x.find((y) => y[0])).map((y) => y[1](y[2]));
const xFnTableLast = (x) => simple_maybe_1.Maybe.of(x
    .slice()
    .reverse()
    .find((y) => y[0])).map((y) => y[1](y[2]));
const xFnTableAny = (x) => simple_maybe_1.Maybe.of(x.filter((y) => y[0])).map((y) => y.map((z) => z[1](z[2])));
const xFnTableN = (n) => (x) => simple_maybe_1.Maybe.of(x.filter((y) => y[0]))
    .map((y) => y.slice(0, n))
    .map((y) => y.map((z) => z[1](z[2])));
const xTableFirst = (x) => simple_maybe_1.Maybe.of(x.find((y) => y[0])).map((y) => y[1]);
const xTableLast = (x) => simple_maybe_1.Maybe.of(x
    .slice()
    .reverse()
    .find((y) => y[0])).map((y) => y[1]);
const xTableAny = (x) => simple_maybe_1.Maybe.of(x.filter((y) => y[0])).map((y) => y.map((z) => z[1]));
const xTableN = (n) => (x) => simple_maybe_1.Maybe.of(x.filter((y) => y[0]))
    .map((y) => y.slice(0, n))
    .map((y) => y.map((z) => z[1]));
const decide = (t, x) => {
    // decide which strategy to use
    const condTable = [
        [x[0].length === 3 && t === 'first', xFnTableFirst],
        [x[0].length === 3 && t === 'last', xFnTableLast],
        [x[0].length === 3 && t === 'any', xFnTableAny],
        [x[0].length === 3 && typeof t === 'number', xFnTableN(t)],
        [t === 'first', xTableFirst],
        [t === 'last', xTableLast],
        [t === 'any', xTableAny],
        [isFinite(t), xTableN(t)]
    ];
    const finder = condTable.find((y) => y[0]);
    return simple_maybe_1.Maybe.of(finder).fork(() => console.error('condition table is broken'), (y) => y[1](x));
};
const Decision = (x) => ({
    map: (f) => Decision(f(x)),
    chain: (f) => f(x),
    ap: (y) => y.map(x),
    inspect: () => `Decision(${x})`,
    join: () => x,
    emit: () => x,
    concat: (o) => o.chain((r) => Decision(x.concat(r)) // @todo should be x as DecisionTable, no?
    ),
    head: () => x[0],
    tail: () => x[x.length - 1],
    isEmpty: () => Boolean(x.length === 0),
    run: (t = 'first') => decide(t, x)
});
const decisionTypeError = (x) => {
    console.error('Decision must be passed parameters that adhere to the documented type. Value that was passed:', x);
    return Decision([[true, null]]);
};
// @todo make much stricter; more of a sniff test right now
const DecisionOf = (x) => Array.isArray(x) &&
    Array.isArray(x[0]) &&
    x[0].length > 1 &&
    x[0].length < 4
    ? Decision(x)
    : decisionTypeError(x);
const exportDecision = {
    of: DecisionOf
};
exports.Decision = exportDecision;
const findIfCond = (a, x) => x.find((y) => (y[0] === a ? true : false));
const queryBTFn = (x) => (a) => simple_maybe_1.Maybe.of(findIfCond(a, x))
    .map((y) => y.slice(-1).pop())
    .fork(_ => {
    console.warn('`if` condition not found: ', a);
    return false;
}, y => Boolean(y));
const BoolTable = (x) => ({
    map: (f) => BoolTable(f(x)),
    chain: (f) => f(x),
    ap: (y) => y.map(x),
    inspect: () => `BoolTable(${x})`,
    join: () => x,
    emit: () => x,
    concat: (o) => o.chain((r) => BoolTable(x.concat(r))),
    head: () => Array.isArray(x) && x.length ? x[0] : [],
    tail: () => Array.isArray(x) && x.length ? x[x.length - 1] : [],
    isEmpty: () => Boolean(!Array.isArray(x) || x.length === 0),
    q: queryBTFn(x),
    query: queryBTFn(x)
});
const boolTableTypeError = (x) => {
    console.error('BoolTable must be passed parameters that adhere to the documented type. Value that was passed:', x);
    return BoolTable([['there was a BoolTable type error', true]]);
};
// @todo make much stricter; more of a sniff test right now
const BoolTableOf = (x) => Array.isArray(x) && Array.isArray(x[0]) && x[0].length === 2
    ? BoolTable(x)
    : boolTableTypeError(x);
const exportBoolTable = {
    of: BoolTableOf
};
exports.BoolTable = exportBoolTable;
//# sourceMappingURL=index.js.map